"use strict";

// TODO
// implementation relies on raw-body thus global Promise to exist

var typeis = require("type-is");
var inflate = require("inflation");
var parser = require("raw-body");
var qs = require("querystring");

var types = {
  json: {
    type_string: ["json", "application/*+json", "application/csp-report"],
    run: function run(req, opts) {
      opts.limit = opts.limit || "1mb";
      return parser(req, opts).then(function (result) {
        return JSON.parse(result);
      });
    }
  },

  form: {
    type_string: ["urlencoded"],
    run: function run(req, opts) {
      opts.limit = opts.limit || "56kb";
      return parser(req, opts).then(function (result) {
        return qs.parse(result);
      });
    }
  },

  text: {
    type_string: ["text"],
    run: function run(req, opts) {
      opts.limit = opts.limit || "1mb";
      return parser(req, opts);
    }
  }
};

var default_opts = function default_opts(opts, req) {
  var len = req.headers["content-length"];
  var encoding = req.headers["content-encoding"] || "identity";
  if (len && encoding === "identity") opts.length = ~~len;
  opts.encoding = opts.encoding || "utf8";
  return opts;
};

module.exports = function (opts) {
  var check = Object.keys(types);
  if (opts === undefined) opts = {};
  check.forEach(function (t) {
    if (opts[t] === undefined) opts[t] = {};
  });

  return function (handler) {
    return function (request) {
      var req = request.req();
      var ctype = request.headers["content-type"];
      // typeis() already does these checks, but do it here
      // for quicker exit
      if (typeis.hasBody(req) === false || ctype === undefined) {
        return handler(request);
      }

      for (var i = 0; i < check.length; i++) {
        var t = check[i];
        if (opts[t] !== false && typeis.is(ctype, types[t].type_string)) {
          return types[t].run(inflate(req), default_opts(opts[t], req)).then(function (body) {
            request.body = body;
            return handler(request);
          });
        }
      }

      return handler(request);
    };
  };
};